unit umain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ComCtrls, ExtCtrls, Spin, BGRAVirtualScreen, BGRABitmap, EpikTimer,
  BGRAText,                                 //LCL font rendering
  BGRAVectorize,                            //vectorized font rendering
  BGRAFreeType, LazFreeTypeFontCollection,  //freetype font rendering
  BGRATextFX, BGRAGradients,                //text effect font rendering
  BGRAGradientScanner; //gradient

type

  { TFMain }

  TFMain = class(TForm)
    BGRAVirtualScreen_Preview: TBGRAVirtualScreen;
    Button_CreateVectorizedFont: TButton;
    Button_Font: TButton;
    CheckBox_OuterOutlineOnly: TCheckBox;
    CheckBox_FreeTypeHinted: TCheckBox;
    CheckBox_Lighting: TCheckBox;
    CheckBox_DropShadow: TCheckBox;
    CheckBox_TextRect: TCheckBox;
    CheckBox_Antialiasing: TCheckBox;
    CheckBox_ClearType: TCheckBox;
    CheckBox_Texture: TCheckBox;
    CheckBox_Underline: TCheckBox;
    CheckBox_Bold: TCheckBox;
    CheckBox_Italic: TCheckBox;
    CheckBox_StrikeOut: TCheckBox;
    FontDialog1: TFontDialog;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    LTime: TLabel;
    Panel_VectorizedFont: TPanel;
    Panel_TextEffect: TPanel;
    Panel_LazFreeType: TPanel;
    Panel_ShadowOutline: TPanel;
    RadioButton_FreeType: TRadioButton;
    RadioButton_TextEffect: TRadioButton;
    RadioButton_LCLRendering: TRadioButton;
    RadioButton_VectorizedFont: TRadioButton;
    SpinEdit_OutlineWidth: TSpinEdit;
    Timer_ShowMs: TTimer;
    TrackBar_EmHeight: TTrackBar;
    TrackBar_Orientation: TTrackBar;
    procedure BGRAVirtualScreen_PreviewRedraw(Sender: TObject; Bitmap: TBGRABitmap);
    procedure Button_CreateVectorizedFontClick(Sender: TObject);
    procedure Button_FontClick(Sender: TObject);
    procedure CheckBox_LightingChange(Sender: TObject);
    procedure CheckBox_TextRectChange(Sender: TObject);
    procedure CheckBoxChange(Sender: TObject);
    procedure CheckBox_LCLFineChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RadioButtonChange(Sender: TObject);
    procedure SpinEdit_OutlineWidthChange(Sender: TObject);
    procedure Timer_ShowMsTimer(Sender: TObject);
    procedure TrackBar_EmHeightChange(Sender: TObject);
  private
    { private declarations }
    FShouldExport: boolean;
    FFontName: string;
    FFreeTypeCollection: TFreeTypeFontCollection;
    FOutlineGradient: TBGRAGradientScanner;
    FEpikTimer: TEpikTimer;
    procedure SaveVectorizedFontIfNeeded(Bitmap: TBGRABitmap);
    procedure ApplySelectedFont(Bitmap: TBGRABitmap);
    procedure ChooseFontRenderer(Bitmap: TBGRABitmap);
  public
    { public declarations }
  end;

var
  FMain: TFMain;

implementation

uses BGRABitmapTypes;

{$R *.lfm}

{ TFMain }

procedure TFMain.BGRAVirtualScreen_PreviewRedraw(Sender: TObject; Bitmap: TBGRABitmap);
const testTxt = 'Portez ce whisky au vieux juge blond qui fume';
var grad: TBGRAGradientScanner;

  procedure DoTextRect(ARect: TRect; txt: string; AAlign: TAlignment; ALayout: TTextLayout);
  var
    textstyle: TTextStyle;
  begin
    {$hints off}
    fillchar(textstyle,sizeof(textstyle),0);
    {$hints on}
    textstyle.Alignment := AAlign;
    textstyle.Layout := ALayout;
    textstyle.Clipping := true;
    textstyle.Wordbreak := true;
    if grad = nil then
      Bitmap.TextRect(ARect,ARect.Left,ARect.Top,txt,textstyle,CSSBlue)
    else
      Bitmap.TextRect(ARect,ARect.Left,ARect.Top,txt,textstyle,grad);
  end;

begin
  ChooseFontRenderer(Bitmap);
  SaveVectorizedFontIfNeeded(Bitmap);
  ApplySelectedFont(Bitmap);

  FEpikTimer.Clear;
  FEpikTimer.Start;
  if CheckBox_Texture.Checked then
    grad := TBGRAGradientScanner.Create(BGRA(0,150,255),BGRA(0,0,0),gtLinear,PointF(0,0),PointF(0,Bitmap.Height/40),True,True)
  else
    grad := nil;

  if CheckBox_TextRect.Checked then
  begin
    DoTextRect(rect(0,0,Bitmap.Width div 3,Bitmap.Height),testTxt, taLeftJustify, tlTop);
    DoTextRect(rect(Bitmap.width div 3,0,2*Bitmap.Width div 3,Bitmap.height),testTxt, taCenter, tlCenter);
    DoTextRect(rect(2*Bitmap.width div 3,0,Bitmap.Width,Bitmap.height),testTxt, taRightJustify, tlBottom);
  end else
  begin
    if grad = nil then
    begin
      Bitmap.TextOut(0,0,testTxt, CSSBlue);
      Bitmap.TextOut(Bitmap.Width Div 2,Bitmap.Height div 2-Bitmap.TextSize('Hg').cy div 2,testTxt, CSSBlue, taCenter);
      Bitmap.TextOut(Bitmap.Width,Bitmap.Height-Bitmap.TextSize('Hg').cy,testTxt, CSSBlue, taRightJustify);
    end else
    begin
      Bitmap.TextOut(0,0,testTxt, grad);
      Bitmap.TextOut(Bitmap.Width Div 2,Bitmap.Height div 2-Bitmap.TextSize('Hg').cy div 2,testTxt, grad, taCenter);
      Bitmap.TextOut(Bitmap.Width,Bitmap.Height-Bitmap.TextSize('Hg').cy,testTxt, grad, taRightJustify);
    end;
  end;
  grad.free;
  FEpikTimer.Stop;
  Timer_ShowMs.Enabled := true;
end;

procedure TFMain.Button_CreateVectorizedFontClick(Sender: TObject);
begin
  FShouldExport := True;
  BGRAVirtualScreen_Preview.DiscardBitmap;
end;

procedure TFMain.Button_FontClick(Sender: TObject);
var fs: TFontStyles;
begin
  FontDialog1.Font.Height := TrackBar_EmHeight.Position * FontEmHeightSign;
  fs := [];
  if CheckBox_Bold.Checked then fs := fs+[fsBold];
  if CheckBox_Italic.Checked then fs := fs+[fsItalic];
  FontDialog1.Font.Style := fs;
  FontDialog1.Font.Name := FFontName;
  if FontDialog1.Execute then
  begin
    CheckBox_Bold.Checked := fsBold in FontDialog1.Font.Style;
    CheckBox_Italic.Checked := fsItalic in FontDialog1.Font.Style;
    TrackBar_EmHeight.Position := FontDialog1.Font.Height * FontEmHeightSign;
    FFontName := FontDialog1.Font.Name;
    BGRAVirtualScreen_Preview.DiscardBitmap;
  end;
end;

procedure TFMain.CheckBox_LightingChange(Sender: TObject);
begin
  RadioButton_TextEffect.Checked := true;
  BGRAVirtualScreen_Preview.DiscardBitmap;
end;

procedure TFMain.CheckBox_TextRectChange(Sender: TObject);
begin
  BGRAVirtualScreen_Preview.DiscardBitmap;
end;

procedure TFMain.CheckBoxChange(Sender: TObject);
begin
  BGRAVirtualScreen_Preview.DiscardBitmap;
end;

procedure TFMain.CheckBox_LCLFineChange(Sender: TObject);
begin
  BGRAVirtualScreen_Preview.DiscardBitmap;
end;

procedure TFMain.FormCreate(Sender: TObject);
begin
  FFontName := 'Arial';
  FFreeTypeCollection := TFreeTypeFontCollection.Create;
  FFreeTypeCollection.AddFolder('.');
  SetDefaultFreeTypeFontCollection(FFreeTypeCollection);
  FOutlineGradient := TBGRAGradientScanner.Create(CSSOrange,CSSYellow,gtLinear,PointF(0,0),PointF(20,20),True,True);
  FEpikTimer := TEpikTimer.Create(nil);
end;

procedure TFMain.FormDestroy(Sender: TObject);
begin
  BGRAVirtualScreen_Preview.Bitmap.FontRenderer := nil;
  SetDefaultFreeTypeFontCollection(nil);
  FFreeTypeCollection.Free;
  FOutlineGradient.Free;
  FEpikTimer.Free;
end;

procedure TFMain.RadioButtonChange(Sender: TObject);
begin
  BGRAVirtualScreen_Preview.DiscardBitmap;
  Button_CreateVectorizedFont.Enabled := RadioButton_VectorizedFont.Checked;
  CheckBox_ClearType.Enabled := RadioButton_FreeType.Checked or RadioButton_LCLRendering.Checked;
  TrackBar_Orientation.Enabled := not RadioButton_FreeType.Checked;
  Panel_LazFreeType.Visible := RadioButton_FreeType.Checked;
  Panel_TextEffect.Visible := RadioButton_TextEffect.Checked;
  Panel_VectorizedFont.Visible := RadioButton_VectorizedFont.Checked;
  Panel_ShadowOutline.Visible := RadioButton_VectorizedFont.Checked or RadioButton_TextEffect.Checked;
end;

procedure TFMain.SpinEdit_OutlineWidthChange(Sender: TObject);
begin
  BGRAVirtualScreen_Preview.DiscardBitmap;
end;

procedure TFMain.Timer_ShowMsTimer(Sender: TObject);
begin
  LTime.Caption := inttostr(round(FEpikTimer.Elapsed*1000))+' ms';
  Timer_ShowMs.Enabled := false;
end;

procedure TFMain.TrackBar_EmHeightChange(Sender: TObject);
begin
  BGRAVirtualScreen_Preview.DiscardBitmap;
end;

procedure TFMain.SaveVectorizedFontIfNeeded(Bitmap: TBGRABitmap);
var
  vf : TBGRAVectorizedFont;
  filename: string;
begin
  if FShouldExport then
  begin
    if Bitmap.FontRenderer is TBGRAVectorizedFontRenderer then
    begin
      vf := TBGRAVectorizedFont.Create;
      vf.Name := Bitmap.FontName;
      vf.Style := Bitmap.FontStyle*[fsBold,fsItalic];
      vf.Resolution := 100;
      vf.NeedGlyphAnsiRange;
      filename := StringReplace(vf.Name,' ','',[rfReplaceAll])+'-';
      if fsBold in vf.Style then filename += 'Bold';
      if fsItalic in vf.Style then filename += 'Italic';
      if [fsBold,fsItalic]*vf.Style = [] then filename += 'Regular';
      vf.SaveGlyphsToFile(filename+'.glyphs');
      vf.Free;
      Bitmap.FontRenderer := TBGRAVectorizedFontRenderer.Create(ExtractFilePath(Application.ExeName));
    end;
    FShouldExport := false;
  end;
end;

procedure TFMain.ApplySelectedFont(Bitmap: TBGRABitmap);
var fs: TFontStyles;
begin
  fs := [];
  if CheckBox_Bold.Checked then fs := fs+[fsBold];
  if CheckBox_Italic.Checked then fs := fs+[fsItalic];
  if CheckBox_Underline.Checked then fs := fs+[fsUnderline];
  if CheckBox_StrikeOut.Checked then fs := fs+[fsStrikeOut];
  Bitmap.FontName:= FFontName;
  Bitmap.FontStyle := fs;
  Bitmap.FontHeight := TrackBar_EmHeight.Position;
  Bitmap.FontOrientation := TrackBar_Orientation.Position*100;
  if CheckBox_Antialiasing.Checked then
  begin
    if CheckBox_ClearType.Checked then
      Bitmap.FontQuality:= fqFineClearTypeRGB
    else
      Bitmap.FontQuality:= fqFineAntialiasing
  end
  else
  begin
    if CheckBox_ClearType.Checked then
      Bitmap.FontQuality:= fqSystemClearType
    else
      Bitmap.FontQuality := fqSystem;
  end;
end;

procedure TFMain.ChooseFontRenderer(Bitmap: TBGRABitmap);
var
  lShader: TPhongShading;
begin
  if RadioButton_LCLRendering.Checked then
    if not (Bitmap.FontRenderer is TLCLFontRenderer) then Bitmap.FontRenderer := nil; //use default

  if RadioButton_TextEffect.Checked then
  begin
    if not (Bitmap.FontRenderer is TBGRATextEffectFontRenderer) then
    begin
      lShader := TPhongShading.Create;
      Bitmap.FontRenderer := TBGRATextEffectFontRenderer.Create(lShader,True);
    end;
    with Bitmap.FontRenderer as TBGRATextEffectFontRenderer do
    begin
      ShaderActive := CheckBox_Lighting.Checked;
      ShaderLightPosition := Point(Bitmap.width div 2,-50);

      ShadowVisible := CheckBox_DropShadow.Checked;
      ShadowColor := BGRABlack;
      ShadowOffset := Point(5,5);
      ShadowRadius := 5;

      OutlineColor := CSSOrange;
      OutlineWidth := SpinEdit_OutlineWidth.Value;
      OutlineVisible := True;
      OuterOutlineOnly:= CheckBox_OuterOutlineOnly.Checked;
      if CheckBox_Texture.Checked then
        OutlineTexture := FOutlineGradient
      else
        OutlineTexture := nil;
    end;
  end;

  if RadioButton_VectorizedFont.Checked then
  begin
    if not (Bitmap.FontRenderer is TBGRAVectorizedFontRenderer) then
      Bitmap.FontRenderer := TBGRAVectorizedFontRenderer.Create(ExtractFilePath(Application.ExeName));
    with Bitmap.FontRenderer as TBGRAVectorizedFontRenderer do
    begin
      ShadowVisible := CheckBox_DropShadow.Checked;
      ShadowColor := BGRABlack;
      ShadowOffset := Point(5,5);
      ShadowRadius := 5;

      ShadowVisible := CheckBox_DropShadow.Checked;
      ShadowColor := BGRABlack;
      ShadowOffset := Point(5,5);
      ShadowRadius := 5;

      OutlineColor := CSSOrange;
      OutlineWidth := SpinEdit_OutlineWidth.Value;
      OuterOutlineOnly:= CheckBox_OuterOutlineOnly.Checked;
      if CheckBox_Texture.Checked then
        OutlineTexture := FOutlineGradient
      else
        OutlineTexture := nil;
    end;
  end;

  if RadioButton_FreeType.Checked then
  begin
    if not (Bitmap.FontRenderer is TBGRAFreeTypeFontRenderer) then
      Bitmap.FontRenderer := TBGRAFreeTypeFontRenderer.Create;
    with Bitmap.FontRenderer as TBGRAFreeTypeFontRenderer do
      FontHinted := Checkbox_FreeTypeHinted.Checked;
  end;
end;

end.

